package com.engine.organization.service.impl;

import com.api.browser.bean.SearchConditionGroup;
import com.api.browser.bean.SearchConditionItem;
import com.cloudstore.eccom.constant.WeaBoolAttr;
import com.cloudstore.eccom.pc.table.WeaTableColumn;
import com.cloudstore.eccom.pc.table.WeaTableOperates;
import com.cloudstore.eccom.result.WeaResultMsg;
import com.engine.common.util.ServiceUtil;
import com.engine.core.impl.Service;
import com.engine.organization.component.OrganizationWeaTable;
import com.engine.organization.entity.column.TableColumnPO;
import com.engine.organization.entity.danikor.param.ChartParam;
import com.engine.organization.entity.danikor.vo.ResourceBackVO;
import com.engine.organization.manager.DanikorOrgChartManager;
import com.engine.organization.mapper.danikor.DanikorResourceMapper;
import com.engine.organization.service.DanikorResourceBackService;
import com.engine.organization.util.MenuBtn;
import com.engine.organization.util.OrganizationFormItemUtil;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.common.StringUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.Util;
import weaver.hrm.appdetach.AppDetachComInfo;
import weaver.hrm.chart.domain.HrmChartSet;
import weaver.hrm.chart.domain.HrmCompanyVirtual;
import weaver.hrm.chart.manager.HrmChartSetManager;
import weaver.hrm.chart.manager.HrmCompanyVirtualManager;
import weaver.hrm.company.CompanyComInfo;
import weaver.hrm.companyvirtual.CompanyVirtualComInfo;
import weaver.systeminfo.SystemEnv;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2023/01/17
 * @version: 1.0
 */
public class DanikorResourceBackServiceImpl extends Service implements DanikorResourceBackService {

    private DanikorResourceMapper getDanikorResourceMapper() {
        return MapperProxyFactory.getProxy(DanikorResourceMapper.class);
    }


    @Override
    public Map<String, Object> listPage(Map<String, Object> params) {
        HrmResourceServiceImpl resourceService = ServiceUtil.getService(HrmResourceServiceImpl.class, user);
        OrganizationWeaTable<ResourceBackVO> table = new OrganizationWeaTable<>(user, ResourceBackVO.class);
        String syncDate = Util.null2String(params.get("syncDate"));
        String sqlForm = "from hrmresource t left join cus_fielddata t0 on t.id = t0.id and t0.scopeid ='-1' left join cus_fielddata t1 on t.id = t1.id and t1.scopeid ='1' left join cus_fielddata t2 on t.id = t2.id and t2.scopeid ='3' left join hrmjobtitles t3 on t.jobtitle=t3.id left join hrmjobactivities t4 on t3.jobactivityid=t4.id left join hrmjobgroups t5 on t4.jobgroupid=t5.id ";
        String sqlWhere = "where t.status < 4 ";
        String cusTableName = "cus_fielddata";
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            sqlWhere += " and t.sync_date = '" + syncDate + "'";
            sqlForm = "from hrmresource_back t left join cus_fielddata_back t0 on t.id = t0.id and t0.scopeid = '-1' and t.sync_date = t0.sync_date left join cus_fielddata_back t1 on t.id = t1.id and t1.scopeid = '1' and t.sync_date = t1.sync_date left join cus_fielddata_back t2 on t.id = t2.id and t2.scopeid = '3' and t.sync_date = t2.sync_date left join hrmjobtitles t3 on t.jobtitle = t3.id left join hrmjobactivities t4 on t3.jobactivityid = t4.id left join hrmjobgroups t5 on t4.jobgroupid = t5.id ";
            cusTableName = "cus_fielddata_back";
        }
        table.setSqlform(sqlForm);
        table.setSqlwhere(sqlWhere);
        table.setOperates(new WeaTableOperates());
        List<WeaTableColumn> weaTableColumnList = new ArrayList<>();
        List<String> fields = new ArrayList<>();
        List<SearchConditionGroup> allConditions = resourceService.getAllConditions();

        //初次使用，无模板初始值
        String columns = "-1_hrm_lastname,-1_hrm_departmentid,-1_hrm_subcompanyid1,-1_hrm_jobtitle,-1_hrm_mobile,-1_hrm_telephone,-1_hrm_managerid,-1_hrm_dsporder";

        List<String> cusFieldNames = getDanikorResourceMapper().getCusColumnByTableName(cusTableName).stream().map(TableColumnPO::getColumnName).collect(Collectors.toList());
        List<String> columnList = Arrays.asList(columns.split(","));
        BigDecimal decimal = new BigDecimal(100 / columnList.size());
        for (SearchConditionGroup allCondition : allConditions) {
            List<SearchConditionItem> items = allCondition.getItems();
            for (SearchConditionItem item : items) {
                String columnName = item.getDomkey()[0];
                String scopeId = columnName.split("_")[0];
                String fieldName = columnName.substring(columnName.lastIndexOf("_") + 1);
                if (columnName.contains("_cus_") && !cusFieldNames.contains(fieldName)) {
                    continue;
                }
                fields.add(resourceService.buildTableSql(columnName) + " as " + resourceService.buildTableSql(columnName).replace(".", "_"));

                WeaTableColumn weaTableColumn = new WeaTableColumn();
                String fieldLabel = getDanikorResourceMapper().getFieldLabel(fieldName, scopeId);
                try {
                    fieldLabel = Util.toScreen(SystemEnv.getHtmlLabelName(Integer.parseInt(fieldLabel), user.getLanguage()), user.getLanguage());
                } catch (Exception e) {
                    new BaseBean().errorLog("数据转换异常", e);
                }
                weaTableColumn.setText(fieldLabel);
                weaTableColumn.setColumn(resourceService.buildTableSql(columnName).replace(".", "_"));

                weaTableColumn.setDisplay(columnList.contains(columnName) ? WeaBoolAttr.TRUE : WeaBoolAttr.FALSE);
                if ("-1_hrm_subcompanyid1".equals(columnName)) {
                    weaTableColumn.setTransmethod("com.engine.organization.transmethod.HrmResourceTransMethod.getCompanyName");
                } else {
                    weaTableColumn.setTransmethod("com.engine.organization.transmethod.HrmResourceTransMethod.getFieldTrueValue");
                }
                weaTableColumn.setOtherpara(columnName);
                weaTableColumn.setWidth(decimal.setScale(2, RoundingMode.HALF_UP).doubleValue() + "%");
                weaTableColumnList.add(weaTableColumn);
            }
        }

        // 增加id字段,跳转人员卡片
        WeaTableColumn weaTableColumn = new WeaTableColumn();
        weaTableColumn.setColumn("id");
        weaTableColumn.setDisplay(WeaBoolAttr.FALSE);

        weaTableColumnList.add(weaTableColumn);
        fields.add("t.id");
        table.setBackfields(StringUtils.join(fields, ","));
        table.setColumns(weaTableColumnList);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        return new HashMap<>(result.getResultMap());
    }

    @Override
    public Map<String, Object> getSearchCondition() {
        Map<String, Object> apiDatas = new HashMap<>();
        List<SearchConditionGroup> addGroups = new ArrayList<>();
        List<SearchConditionItem> conditionItems = new ArrayList<>();
        SearchConditionItem syncDate = OrganizationFormItemUtil.datePickerItem(user, 2, 16, false, 2, "查询时间", "syncDate");
        conditionItems.add(syncDate);
        addGroups.add(new SearchConditionGroup("高级搜索条件", true, conditionItems));
        apiDatas.put("conditions", addGroups);
        return apiDatas;
    }

    @Override
    public Map<String, List<MenuBtn>> getHasRight() {
        Map<String, List<MenuBtn>> btnDatas = new HashMap<>();
        ArrayList<MenuBtn> rightMenuList = new ArrayList<>();
        // 显示列定制
        rightMenuList.add(MenuBtn.rightMenu_btnColumn());
        btnDatas.put("rightMenu", rightMenuList);
        return btnDatas;
    }

    @Override
    public Map<String, Object> getSummaryData(Map<String, Object> params) {
        HashMap<String, Object> returnMap = new HashMap<>();
        String companyTable = "hrmsubcompany";
        String departmentTable = "hrmdepartment";
        String resourceTable = "hrmresource";
        String syncDate = Util.null2String(params.get("syncDate"));
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            companyTable += "_back";
            departmentTable += "_back";
            resourceTable += "_back";
        } else {
            syncDate = null;
        }
        returnMap.put("subcompany", getDanikorResourceMapper().summaryCompanyData(companyTable, syncDate));
        returnMap.put("department", getDanikorResourceMapper().summaryDepartmentData(departmentTable, syncDate));
        returnMap.put("person", getDanikorResourceMapper().summaryResourceData(resourceTable, syncDate));
        return returnMap;
    }

    @Override
    public String getOrgChartData(HttpServletRequest request, Map<String, Object> params) {
        CompanyVirtualComInfo CompanyVirtualComInfo = new CompanyVirtualComInfo();
        CompanyComInfo CompanyComInfo = new CompanyComInfo();
        AppDetachComInfo AppDetachComInfo = new AppDetachComInfo();
        boolean isOutCustomer = AppDetachComInfo.isOutCustomer("" + user.getUID());//是否为外部用户
        String sorgid = Util.null2String((params.get("arg0")));
        int shownum = StringUtil.parseToInt(Util.null2String(params.get("arg1")), 1);
        String showName = Util.null2String(params.get("arg2"));

        int showtype = Util.getIntValue(Util.null2String(params.get("showtype")), 0);
        Map beanMap = new HashMap<>();
        HrmChartSetManager HrmChartSetManager = new HrmChartSetManager();
        beanMap.put("is_sys", 1);
        HrmChartSet bean = HrmChartSetManager.get(beanMap);
        if (bean != null) {
            showtype = bean.getShowType();
            shownum = bean.getShowNum();
        }
        shownum = shownum <= 0 ? 1 : shownum;
        if (0 == showtype) shownum = 10000;

        if (CompanyComInfo.next()) {
            showName = CompanyComInfo.getCompanyname();
        }
        if (isOutCustomer) {
            sorgid = "-10000";
            showName = CompanyVirtualComInfo.getVirtualType(sorgid);
        }

        boolean isPOrg = Boolean.parseBoolean(Util.null2s(Util.null2String(params.get("arg3")), "true"));
        String arg4 = Util.null2s(Util.null2String(params.get("arg4")), "8");
        String arg5 = Util.null2String(params.get("arg5"));
        String arg6 = Util.null2String(params.get("arg6"));
        String arg7 = Util.null2String(params.get("arg7"));
        String arg8 = Util.null2String(params.get("arg8"));
        String arg9 = Util.null2String(params.get("arg9"));
        String arg10 = Util.null2String(params.get("arg10"));
        String arg11 = Util.null2s(Util.null2String(params.get("arg11")), ";;P");

        HrmCompanyVirtualManager manager = new HrmCompanyVirtualManager();

        HrmCompanyVirtual cvBean = null;
        if (!isPOrg) {
            cvBean = manager.get(sorgid);
            showName = cvBean != null ? cvBean.getVirtualtype() : "";
        }

        //初始化tlevel
        new com.api.hrm.util.ServiceUtil().initOrgLevel();

        Map<String, Object> map = new HashMap<>();
        map.put("status", arg4);
        map.put("docStatus", arg5);
        map.put("customerType", arg6);
        map.put("customerStatus", arg7);
        map.put("workType", arg8);
        map.put("projectStatus", arg9);

        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("cmd", arg10);
        requestMap.put("sorgid", sorgid);

        showName = StringUtil.vString(StringUtil.toScreen(Util.formatMultiLang(showName, "" + user.getLanguage())));

        DanikorOrgChartManager chartManager = new DanikorOrgChartManager();
        chartManager.setShowNum(shownum);
        chartManager.init(user, request, map, arg11, isPOrg, cvBean, requestMap);
        String showCount = String.valueOf(chartManager.getCompanyResourceCount());
        String showTitle = showName + SystemEnv.getHtmlLabelNames(chartManager.getLabelIndex() + ",523", user.getLanguage()) + showCount;

        String defaultHref = weaver.general.GCONST.getContextPath() + "/spa/hrm/index_mobx.html#/main/hrm/orgStaff?_fromURL=HrmResourceSearchResult&from=hrmorg&virtualtype=" + sorgid;
        return "[" + "{\"id\":\"0\", \"pid\":\"\", \"type\":\"company\", \"name\":\"" + StringUtil.vString(showName, 12) + "\", \"title\":\"" + showName +
                "\", \"num\":\"" + showCount + "\", \"nTitle\":\"" + showTitle + "\", \"oDisplay\":\"none\", \"subRCount\":\"0\", \"subTitle\":\"\", \"cOnclick\":\"" + defaultHref + "\", \"sOnclick\":\"\"}" + chartManager.getData() +
                "]";
    }

    @Override
    public List<ChartParam> getOrgChartData(Map<String, Object> params) {
        String syncDate = Util.null2String(params.get("date"));
        List<ChartParam> chartParamList = new ArrayList<>();
        String sorgid = Util.null2String((params.get("arg0")));
        String defaultHref = weaver.general.GCONST.getContextPath() + "/spa/hrm/index_mobx.html#/main/hrm/orgStaff?_fromURL=HrmResourceSearchResult&from=hrmorg&virtualtype=" + sorgid;
        CompanyComInfo CompanyComInfo = new CompanyComInfo();

        if (CompanyComInfo.next()) {
            chartParamList.add(ChartParam.builder().id("0").pid("").type("company").name(CompanyComInfo.getCompanyname()).title(CompanyComInfo.getCompanyname()).num("0").nTitle("").oDisplay("none").subRCount("0").subTitle("").cOnclick(defaultHref).sOnclick("").build());
        }

        String subCompanySql;
        String departmentSql;
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            subCompanySql = "select t.id, t.subcompanyname, t.supsubcomid, ( select distinct 1 as hasnext from hrmsubcompany_back s where (s.supsubcomid = t.id and (s.CANCELED IS NULL OR s.CANCELED != '1') and s.sync_date = '" + syncDate + "') ) as hassubnext, ( select distinct 1 as hasnext from HrmDepartment_back d where (d.subcompanyid1 = t.id and (d.CANCELED IS NULL OR d.CANCELED != '1') and d.sync_date = '" + syncDate + "')) as hasdeptnext, t.tlevel from hrmsubcompany_back t where (t.canceled IS NULL OR t.canceled != '1') and t.id != t.supsubcomid and tlevel <= 10000 and sync_date = '" + syncDate + "' order by t.supsubcomid, t.showorder, t.subcompanyname";
            departmentSql = "select t.id, t.departmentname, t.subcompanyid1, isnull(t.supdepid,0) as supdepid, (case when t.supdepid != 0 and ( select COUNT(id) from hrmdepartment_back where id = t.supdepid and sync_date = '" + syncDate + "')= 0 then 0 else 1 end) as isExist, ( select distinct 1 as hasnext from hrmdepartment_back d where d.supdepid = t.id and (CANCELED IS NULL OR CANCELED != '1') and sync_date = '" + syncDate + "') as hasnext, t.tlevel from hrmdepartment_back t where t.id != ISNULL(t.supdepid, 0) and tlevel <= 10000 and (t.canceled IS NULL OR t.canceled != '1') and (t.canceled IS NULL OR t.canceled != '1') and sync_date = '" + syncDate + "' order by t.subcompanyid1 asc , t.supdepid asc , t.showorder asc, t.departmentname asc";
        } else {
            subCompanySql = "select t.id,t.subcompanyname,t.supsubcomid,(select distinct 1 as hasnext from hrmsubcompany s where (s.supsubcomid=t.id and (s.CANCELED IS NULL OR s.CANCELED !='1')) ) as hassubnext,(select distinct 1 as hasnext from HrmDepartment d where (d.subcompanyid1=t.id and (d.CANCELED IS NULL OR d.CANCELED !='1')) ) as hasdeptnext,t.tlevel from hrmsubcompany t where (t.canceled IS NULL OR t.canceled !='1') and t.id != t.supsubcomid  and tlevel <= 10000 order by t.supsubcomid,t.showorder,t.subcompanyname";
            departmentSql = "select t.id,t.departmentname,t.subcompanyid1,isnull(t.supdepid,0) as supdepid,(case when t.supdepid != 0 and (select COUNT(id) from hrmdepartment where id = t.supdepid)=0 then 0 else 1 end)  as isExist,(select distinct 1 as hasnext from hrmdepartment d where d.supdepid=t.id and (CANCELED IS NULL OR CANCELED !='1')) as hasnext,t.tlevel from hrmdepartment t where  t.id != ISNULL(t.supdepid,0)  and tlevel <= 10000 and (t.canceled IS NULL OR t.canceled !='1')  order by t.subcompanyid1 asc , t.supdepid asc , t.showorder asc, t.departmentname asc";
        }

        RecordSet rs = new RecordSet();
        //rs.execute(subCompanySql);
        //while (rs.next()) {
        //    int subHasNext = rs.getInt("hassubnext");
        //    int deptHasNext = rs.getInt("hasdeptnext");
        //    ChartParam chartParam = new ChartParam();
        //    chartParam.setId(rs.getString("id"));
        //    chartParam.setPid(rs.getString("supsubcomid"));
        //    chartParam.setType("subcompany");
        //    chartParam.setName(rs.getString("subcompanyname"));
        //    chartParam.setTitle(chartParam.getName());
        //    chartParam.setNum("0");
        //    chartParam.setNTitle("");
        //    chartParam.setODisplay("none");
        //    chartParam.setSubRCount("0");
        //    chartParam.setSubTitle("");
        //    chartParam.setHasChild(subHasNext == 1 || deptHasNext == 1 ? "true" : "false");
        //    chartParam.setNeedPlus("false");
        //    chartParam.setCOnclick("");
        //    chartParam.setSOnclick("");
        //    chartParamList.add(chartParam);
        //}

        // 处理部门数据
        rs.execute(departmentSql);
        while (rs.next()) {
            int hasNext = rs.getInt("hasnext");
            ChartParam chartParam = new ChartParam();
            chartParam.setId(rs.getString("id"));
            chartParam.setPid(rs.getString("supdepid"));
            chartParam.setType("dept");
            chartParam.setName(rs.getString("departmentname"));
            chartParam.setTitle(chartParam.getName());
            chartParam.setNum("0");
            chartParam.setNTitle("");
            chartParam.setODisplay("none");
            chartParam.setSubRCount("0");
            chartParam.setSubTitle("");
            chartParam.setHasChild(hasNext == 1 ? "true" : "false");
            chartParam.setNeedPlus("false");
            chartParam.setCOnclick("");
            chartParam.setSOnclick("");
            chartParamList.add(chartParam);
        }

        return chartParamList;

    }

    @Override
    public List<ChartParam> getResourceChartData(Map<String, Object> params) {
        String syncDate = Util.null2String(params.get("date"));
        List<ChartParam> chartParamList = new ArrayList<>();
        // 查询安全级别大于等于40的人员
        String sql;
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            sql = "select a.id, a.managerid , a.lastname , b.jobtitlename, ( select count(1) from HrmResource_back where managerid = a.id and sync_date = '" + syncDate + "') subordinate from HrmResource_back a left join HrmJobTitles b on a.jobtitle = b.id where seclevel >= 40 and sync_date = '" + syncDate + "'";
        } else {
            sql = "select a.id, a.managerid , a.lastname , b.jobtitlename, ( select count(1) from HrmResource where managerid = a.id) subordinate from HrmResource a left join HrmJobTitles b on a.jobtitle = b.id where seclevel >= 40";
        }

        RecordSet rs = new RecordSet();
        rs.execute(sql);
        while (rs.next()) {
            String lastName = rs.getString("lastname");
            String jobTitleName = rs.getString("jobtitlename");
            int subordinate = rs.getInt("subordinate");
            String managerId = rs.getString("managerid");
            ChartParam chartParam = new ChartParam();
            chartParam.setId(rs.getString("id"));
            chartParam.setPid(managerId);
            chartParam.setType((StringUtils.isBlank(managerId) || "0".equals(managerId)) ? "company" : "subcompany");
            chartParam.setName(StringUtils.isBlank(jobTitleName) ? lastName : lastName + "(" + jobTitleName + ")");
            chartParam.setTitle(chartParam.getName());
            chartParam.setNum("0");
            chartParam.setNTitle("");
            chartParam.setODisplay("none");
            chartParam.setSubRCount("0");
            chartParam.setSubTitle("");
            chartParam.setHasChild(subordinate > 0 ? "true" : "false");
            chartParam.setNeedPlus("false");
            chartParam.setCOnclick("/spa/hrm/index_mobx.html#/main/hrm/card/cardInfo/" + chartParam.getId());
            chartParam.setSOnclick("");
            chartParamList.add(chartParam);
        }

        return chartParamList;

    }
}
