package com.engine.organization.web;

import com.alibaba.fastjson.JSON;
import com.engine.common.util.ParamUtil;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.entity.danikor.param.ChartParam;
import com.engine.organization.util.response.ReturnResult;
import com.engine.organization.wrapper.DanikorResourceBackWrapper;
import org.apache.commons.collections.CollectionUtils;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.User;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author:dxfeng
 * @createTime: 2023/01/28
 * @version: 1.0
 */
public class DanikorResourceBackController {

    public DanikorResourceBackWrapper getDanikorResourceBackWrapper(User user) {
        return ServiceUtil.getService(DanikorResourceBackWrapper.class, user);
    }

    /**
     * 获取list列表
     *
     * @param request
     * @param response
     * @return
     */
    @GET
    @Path("/listPage")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult listPage(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            return ReturnResult.successed(getDanikorResourceBackWrapper(user).listPage(map));
        } catch (Exception e) {
            return ReturnResult.exceptionHandle(e);
        }
    }

    @GET
    @Path("/getSearchCondition")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getSearchCondition(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getDanikorResourceBackWrapper(user).getSearchCondition());
        } catch (Exception e) {
            return ReturnResult.exceptionHandle(e);
        }
    }

    @GET
    @Path("/getHasRight")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getHasRight(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getDanikorResourceBackWrapper(user).getHasRight());
        } catch (Exception e) {
            return ReturnResult.exceptionHandle(e);
        }
    }

    @GET
    @Path("/getSummaryData")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getSummaryData(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            return ReturnResult.successed(getDanikorResourceBackWrapper(user).getSummaryData(map));
        } catch (Exception e) {
            return ReturnResult.exceptionHandle(e);
        }
    }

    @POST
    @Path("/getOrgChartData")
    @Produces(MediaType.APPLICATION_JSON)
    public Map<String, Object> getOrgChartData(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            String orgChartData = getDanikorResourceBackWrapper(user).getOrgChartData(map);
            returnMap.put("data", orgChartData);
            returnMap.put("status", "1");
        } catch (Exception e) {
            returnMap.put("status", "-1");
            returnMap.put("message", "发生未知错误，请与管理员联系");
        }
        return returnMap;
    }

    @POST
    @Path("/getResourceChartData")
    @Produces(MediaType.APPLICATION_JSON)
    public Map<String, Object> getResourceChartData(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            List<ChartParam> resourceChartData = getDanikorResourceBackWrapper(user).getResourceChartData(map);
            returnMap.put("data", CollectionUtils.isEmpty(resourceChartData) ? null : JSON.toJSONString(resourceChartData));
            returnMap.put("status", "1");
        } catch (Exception e) {
            returnMap.put("status", "-1");
            returnMap.put("message", "发生未知错误，请与管理员联系");
        }
        return returnMap;
    }
}
